// Copyright (c) 2023 Dominic Masters
// 
// This software is released under the MIT License.
// https://opensource.org/licenses/MIT

#pragma once
#include "assert/assert.hpp"

namespace Dawn {
  class StateOwner;

  class IStateEvent {
    protected:
      virtual void _stateOwnerDestroyed(StateOwner *owner) = 0;

      friend class StateOwner;
  };

  template<typename...A>
  class StateEvent : public IStateEvent {
    protected:
      void _stateOwnerDestroyed(StateOwner *owner) override {
        this->_eventListeners.erase(owner);
      }

    public:
      std::map<StateOwner*, std::vector<std::function<void(A...)>>> _eventListeners;

      /**
       * Invokes the event and emits to all of the listeners.
       * 
       * @param args Arguments for this event to pass to the listeners.
       */
      void invoke(A... args) {
        auto itListeners = this->_eventListeners.begin();
        while(itListeners != this->_eventListeners.end()) {
          auto itLists = itListeners->second.begin();
          while(itLists != itListeners->second.end()) {
            (*itLists)(args...);
            ++itLists;
          }
          ++itListeners;
        }
      }
  };
}